setOldClass("unit")
setOldClass("viewport")

setClass("hexVP",
         representation(hexVp="viewport",mar="unit",
                        fig="unit",plt="unit",
                        xscale="numeric", yscale="numeric")
         )



hexViewport <-
    function(x, offset = unit(0,"inches"), mar = NULL,
	     xbnds = x@xbnds, ybnds = x@ybnds, newpage = FALSE)
{
    if(!is(x,"hexbin"))
	stop("first argument must be a hexbin object")
    stopifnot(is.unit(offset))

    hvp <- new("hexVP")

    if (newpage)
	grid.newpage()

    if(is.null(mar)) {
	mar <- unit(0.1 + c(5,4,4,2),"lines")
    }
    else{
	if(!is.unit(mar)) stop("'mar' must be specified in units")
	if(length(mar) == 1)
	    mar <- rep(mar, 4)
	else if(length(mar) != 4) stop("'mar' must have length 1 or 4")
    }
    ## in both cases
    mai <- as.numeric(convertUnit(mar, "inches"))

    vpin <- c(convertWidth (unit(1,"npc"),"inches"),
	      convertHeight(unit(1,"npc"),"inches"))
    fig <- c(as.numeric(convertUnit(unit(vpin[1],"inches") - offset,"inches")),
	     vpin[2])
    pin <- c(fig[1]-mai[2]-mai[4],
	     fig[2]-mai[1]-mai[3])
    xsize <- pin[1]
    ysize <- pin[2]
    ##
    if(x@shape * xsize <= ysize) {
	center <- (ysize - x@shape * xsize)/2
	mai[1] <- mai[1] + center
	mai[3] <- mai[3] + center
	ysize <- x@shape * xsize
    }
    else {
	center <- (xsize - ysize/x@shape)/2
	mai[2] <- mai[2] + center
	mai[4] <- mai[4] + center
	xsize <- ysize/x@shape
    }
    fig <- c(pin[1]+mai[2]+ mai[4],fig[2])
    mar <- c(convertUnit(unit(mai[1],"inches"),"lines"),
	     convertUnit(unit(mai[2],"inches"),"lines"),
	     convertUnit(unit(mai[3],"inches"),"lines"),
	     convertUnit(unit(mai[4],"inches"),"lines"))
    #browser()
    pin <- c(fig[1]-(mai[2] + mai[4]),
	     fig[2]-(mai[1] + mai[3]))
    margins <- rep(as.numeric(mar), length.out = 4)
    wd <- convertUnit(unit(pin[1],"inches"),"npc")
    ##(unit(sum(margins[c(2, 4)]), "lines") +
    ##			    convertUnit(unit(legend,"inches"),"lines"))
    addBit <- function(bnds, f = 0.05) bnds + c(-f, f) * diff(bnds)
    hvp@hexVp<-viewport(x = unit(margins[2], "lines"),
                 width = wd,
                 y = unit(margins[1], "lines"),
                 height = unit(1, "npc") - unit(sum(margins[c(1,3)]), "lines"),
                 xscale = addBit(xbnds),
                 yscale = addBit(ybnds),
                 default.unit = "npc", just = c("left", "bottom"))
    hvp@mar <- unit(mar,"lines")
    hvp@xscale <- addBit(xbnds)
    hvp@yscale <- addBit(ybnds)
    hvp@fig <-convertUnit(unit(fig,"inches"),"npc")
    hvp@plt <-convertUnit(unit(pin,"inches"),"npc")
    #hvp@leg <-convertUnit(offset,"npc")
    hvp
}

#setGeneric("grid:::pushViewport")

#setMethod("pushViewport", signature(x="hexVP"), function(hvp)
#          {
#            pushViewport(hvp@hexVp)
#
#          })


pushHexport<-function(hvp){pushViewport(hvp@hexVp)}

#setMethod("push",signature("hexVP"),"pushHexport")

setGeneric("getMargins", function(hvp, ret.unit="npc", numeric=FALSE)
           standardGeneric("getMargins"))
setMethod("getMargins","hexVP",
          function(hvp, ret.unit="npc", numeric=FALSE){
            mar <- convertUnit(hvp@mar,ret.unit)
            if(numeric)as.numeric(mar)
            else mar
          })

setGeneric("getPlt", function(hvp, ret.unit="npc", numeric=FALSE)
           standardGeneric("getPlt"))
setMethod("getPlt",signature("hexVP"),
          function(hvp, ret.unit="npc", numeric=FALSE){
            plt <- convertUnit(hvp@plt,ret.unit)
            if(numeric)as.numeric(plt)
            else plt
          })

setGeneric("getFig", function(hvp, ret.unit="npc", numeric=FALSE)
           standardGeneric("getFig"))
setMethod("getFig",signature("hexVP"),
          function(hvp, ret.unit="npc", numeric=FALSE){
            fig <- convertUnit(hvp@fig,ret.unit)
            if(numeric)as.numeric(fig)
            else fig
          })

setGeneric("getXscale", function(hvp)standardGeneric("getXscale"))
setMethod("getXscale",signature("hexVP"),function(hvp){hvp@xscale})

setGeneric("getYscale", function(hvp)standardGeneric("getYscale"))            
setMethod("getYscale",signature("hexVP"),function(hvp){hvp@yscale})          
